<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xml="http://www.w3.org/XML/1998/namespace" xmlns:as="http://www.oasis-open.org/committees/cam"  xmlns:camed="http://jcam.org.uk/editor"
	xmlns:xsd="http://www.w3.org/2001/XMLSchema"
	xmlns:look="http://jcam.org.uk/LookupLists">
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: David RR Webber, September, 2009 -->
	<!--
		Purpose  : Utility for use with ERwin models. Parses input CAM template file and flags for removal the pieces that are model artifacts; renames model code detail parents.
		Inputs: CAM template.
		Original : 09/23/2009
		Initials : drrw
		Description : Parses input CAM template file and refactors into new CAM template.
		                     Uses default CAM template format, including any extensions.
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
		Updated  - Inits - Change
		03/03/10   drrw   Initial release 
		03/03/10   drrw   Added annotations item path change check
		03/05/10   drrw   Fix outputting of rules and annotations path updating
		03/06/10   drrw   Make parent / child name matching check for _CODE, _COUNT items only
		03/07/10   drrw   Annotations check in structureAnno for CODE_DETAIL
		03/08/10   drrw   Process substitutiongroup annotations for CODE_DETAIL
		03/10/10   drrw   Skip @ref items in _CODE_DETAIL parents
		03/10/10   drrw   Remove duplicate BaseRef items in all structure constructs
		03/12/10   drrw   Add type to annotation of CODE_DETAIL
		03/12/10   drrw   Skip children of removed parent element
		03/22/10   drrw   Make all nillable items optional instead
		05/19/10   drrw   Non-numeric elements with decimal data type
		05/19/10   drrw   Add new structure component reordering step
		05/27/10   drrw   Refactor rules and annotation XPaths if structure item inserted during reordering
		06/02/10   drrw   Adjust XPath for rules and annotations if abstract parent element in re-ordering is inserted
		09/09/10   drrw   Passed parameter reporting in messages
	-->
	<xsl:param name="inputFilePath"></xsl:param>
	<xsl:param name="reorderFilePath"></xsl:param>

	<xsl:output method="xml" indent="yes"/>
	<xsl:variable name="xslt-version">1.13</xsl:variable>
	<xsl:variable name="nlchar" select="'&#xA;'"/>
	<xsl:variable name="debug">0</xsl:variable><!-- 0, 1, 2, 3, 4 or 5 -->
	<xsl:variable name="rootElement" select="as:CAM/as:AssemblyStructure/as:Structure/*[1]/name()"/>
	
	<xsl:variable name="namespaceLookup">
		<namespaces>
		<xsl:for-each select=".//namespace::*">
			<xsl:if test="not(name()='w2e')">
				<namespace name="{name()}" select="{.}"/>
			</xsl:if>
		</xsl:for-each>
		</namespaces>
	</xsl:variable>
	
	<xsl:variable name="itemReorder" select="document($reorderFilePath)"/><!-- Load item reordering map file  -->
	
	<xsl:variable name="itemAnnotations">
		<!-- xsl:message>anno inputFilePath: <xsl:value-of select="$inputFilePath"/></xsl:message -->
		<xsl:choose>
			<xsl:when test="//as:Extension"><xsl:copy-of select="//as:Extension"/></xsl:when>
			<xsl:when test="//as:Include/as:Extension"><xsl:copy-of select="//as:Include/as:Extension"/></xsl:when>
			<xsl:otherwise>
				<xsl:variable name="annoIncludes"><xsl:copy-of select="//as:include"/></xsl:variable>
				<xsl:for-each select="$annoIncludes/as:include">
					<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/><xsl:value-of select="."/></xsl:variable>
					<xsl:if test="contains(.,'.anno')"><xsl:copy-of select="document(replace(replace($filePath,'C:\\','\\'),'\\','/'))"/></xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="refactoredStructure">
		<xsl:apply-templates select="//as:AssemblyStructure/as:Structure" mode="structure"/>
	</xsl:variable>
	
	<xsl:variable name="parsedStructure">
		<xsl:element name="as:AssemblyStructure"><!-- output the expanded Structure details -->
			<xsl:for-each select="//as:AssemblyStructure/@*">
				<xsl:attribute name="{name()}" select="."/>
			</xsl:for-each>
			<xsl:choose>
				<xsl:when test="count($itemReorder//as:Reordering/as:Group) &gt; 0">
					<xsl:message> with reordering <xsl:value-of select="count($itemReorder//as:Reordering/as:Group)"/> groups  <xsl:if test="$debug &gt; 0"><xsl:value-of select="$itemReorder//as:Reordering/as:Group/child::*[1]/name()"/></xsl:if></xsl:message>
					<xsl:apply-templates select="$refactoredStructure" mode="reorder"/>
				</xsl:when>
				<xsl:otherwise><xsl:copy-of select="$refactoredStructure"/></xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:variable>
	
	<xsl:variable name="newRules">
		<xsl:apply-templates select="$refactoredStructure/as:Structure" mode="structureRules"/>
	</xsl:variable>
	
	<xsl:variable name="newAnno">
		<xsl:apply-templates select="$refactoredStructure/as:Structure" mode="structureAnno"/>
	</xsl:variable>
	
	<xsl:strip-space elements="*"/>
	
	<xsl:template match="/">
		<xsl:variable name="templateNS" select=".//namespace::*"/>
		<xsl:message>ERwin model refactor starting <xsl:value-of select="dateTime(current-date(),current-time())"/></xsl:message>
		<xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
		<xsl:message>From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/></xsl:message>
		<xsl:if test="not($inputFilePath='')"><xsl:message>Input File Path: <xsl:value-of select="$inputFilePath"/></xsl:message></xsl:if>
		<xsl:if test="not($reorderFilePath='')"><xsl:message>Using reordering: <xsl:value-of select="$reorderFilePath"/></xsl:message></xsl:if>
		
		<as:CAM xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:xml="http://www.w3.org/XML/1998/namespace" CAMlevel="{as:CAM/@CAMlevel}" version="{as:CAM/@version}">
			<xsl:for-each select=".//namespace::*">
				<xsl:if test="not(name()='w2e')">
					<xsl:namespace name="{name()}" select="."/>
				</xsl:if>
			</xsl:for-each>

<xsl:comment xml:space="preserve">
	CAM ERwin refactor 
	Version: <xsl:value-of select="$xslt-version"/>
	Date: <xsl:value-of select="dateTime(current-date(),current-time())"/>
	From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/>
	<xsl:if test="not($reorderFilePath='')">Using reordering: <xsl:value-of select="$reorderFilePath"/></xsl:if>

</xsl:comment>

			<xsl:message>Writing structure</xsl:message>
			<xsl:message> (path trace diagnostics)</xsl:message>
			<xsl:apply-templates select="//as:Header"/>
			
			<xsl:apply-templates mode="outputStructure" select="$parsedStructure"/>
			<xsl:message>structure completed</xsl:message>
			<xsl:message>Writing rules</xsl:message>
			<xsl:apply-templates select="//as:BusinessUseContext"/>
			<xsl:message>completed rules</xsl:message>
			<!-- xsl:message>annotations: [<xsl:copy-of select="$itemAnnotations"/>]</xsl:message -->
			
			<xsl:message>Writing annotations</xsl:message>
			<xsl:choose>
				<xsl:when test="//as:Extension"><xsl:apply-templates select="//as:Extension"/></xsl:when>
				<xsl:when test="//as:include"><xsl:apply-templates select="//as:include"/></xsl:when>
				<xsl:otherwise>
					<as:Extension name="uk.org.jcam.camed.extensions.StructureAnnotations">
						<xsl:copy-of select="$newAnno/as:MoreAnno/*"/>
					</as:Extension>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:message>completed annotations</xsl:message>
			<!-- xsl:apply-templates select="//as:include"/ --><!-- disable for now, CAMeditor will recreate includes on save -->			
		</as:CAM>
		<xsl:message>CAM ERwin refactor finished processing.</xsl:message>
	</xsl:template>

	<xsl:template match="as:Header">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:AssemblyStructure">
		<xsl:copy>
			<xsl:apply-templates select="as:Structure"/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="as:BusinessUseContext">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="as:Extension">
		<as:Extension name="{@name}">
			<xsl:apply-templates/>
			<xsl:copy-of select="$newAnno/as:MoreAnno/*"/>
		</as:Extension>
	</xsl:template>
	
	<xsl:template match="camed:annotation">
		<!-- Check to see if parent node renamed -->
		<xsl:choose>
			<xsl:when test="contains(./@item,'_CODE/') or(contains(./@item,'_COUNT/'))">
				<xsl:variable name="parentPath" select="concat(substring-before(substring-after(./@item,'//'),'/'),'_DETAIL')"/>
				<xsl:variable name="matchNode" select="count($refactoredStructure/as:Structure/AllRootLevelTypeDefinitions/node()[name()=$parentPath])"/>
				<xsl:choose>
					<xsl:when test="$matchNode &gt; 0">
						<camed:annotation item="{replace(@item,concat(replace($parentPath,'_DETAIL',''),'/'),concat($parentPath,'/'))}">
							<xsl:apply-templates/>
						</camed:annotation>
					</xsl:when>
					<xsl:otherwise>
						<camed:annotation item="{@item}">
							<xsl:apply-templates/>
						</camed:annotation>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<!-- xsl:when test=""></xsl:when> check here for parent node matching front part of XPath - if not - replace that node in path !!! -->
			
			<xsl:otherwise><!-- No change so write it out as is -->
			       <xsl:choose>		       	
				<xsl:when test="contains(replace(@item,'//','?'),'/')">
					<xsl:variable name="parentPath" select="substring-before(substring-after(./@item,'//'),'/')"/>
					<xsl:variable name="nodeName" select="substring-after(./@item,concat($parentPath,'/'))"/>
					<xsl:variable name="rootNode" select="count($parsedStructure/as:AssemblyStructure/as:Structure/node()[name()=$parentPath])"/>
					<xsl:variable name="parentNode" select="count($parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath])"/>
					<xsl:variable name="childNodes" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/node()/name()"/>
					<xsl:variable name="childNode">
						<xsl:for-each select="$childNodes">
							<xsl:if test=".=$nodeName"><xsl:value-of select="$nodeName"/></xsl:if>
						</xsl:for-each>
					</xsl:variable>
					<!-- xsl:variable name="childNode" select="count($refactoredStructure/as:Structure[*/*/node()/name()=$nodeName][*/node()/name()=$parentPath])"/ -->
					<xsl:choose>
						<xsl:when test="$parentPath = $nodeName"/><!-- ignore child and parent same path -->
						<xsl:when test="$parentNode &gt; 0 and($childNode='')">
							<xsl:variable name="findNodes" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/*/node()/name()"/>
							<xsl:variable name="findNodesContent" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/*/node()"/>
							<xsl:variable name="findNode">
								<xsl:for-each select="$findNodes">
									<xsl:variable name="this" select="position()"/>
									<xsl:if test=".=$nodeName"><xsl:value-of select="$nodeName"/> - <xsl:value-of select="$findNodesContent[position()=$this]/text()"/> - <xsl:value-of select="$this"/> + <xsl:value-of select="$findNodesContent[position()=$this]/parent::node()/name()"/></xsl:if>
								</xsl:for-each>
							</xsl:variable>
						           <xsl:choose>
							    <xsl:when test="not($findNode='') and(contains($findNode,'%'))">
								<xsl:message> new parent node of <xsl:value-of select="@item"/> is <xsl:value-of select="substring-after($findNode,'+ ')"/></xsl:message>
								<camed:annotation item="{concat('//',substring-after($findNode,'+ '),'/',$nodeName)}">
									<xsl:apply-templates/>
								</camed:annotation>
							    </xsl:when>
							    <xsl:otherwise>
								<camed:annotation item="{@item}">
									<xsl:apply-templates/>
								</camed:annotation>
							    </xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise>
							<camed:annotation item="{@item}">
								<xsl:apply-templates/>
							</camed:annotation>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
			       	<xsl:otherwise>
			       		<camed:annotation item="{@item}">
			       			<xsl:apply-templates/>
			       		</camed:annotation>
			       	</xsl:otherwise>
			       	</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="camed:documentation">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="look:LookupList">
		<xsl:copy>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="look:item">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:Structure">
		<as:Structure ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates mode="structure"/>
		</as:Structure>
	</xsl:template>
	
	<xsl:template match="as:Structure" mode="structureRules">
		<as:MoreRules ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureRules"/>
		</as:MoreRules>
	</xsl:template>

	<xsl:template match="as:Structure" mode="structureAnno">
		<as:MoreAnno ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureAnno"/>
		</as:MoreAnno>
	</xsl:template>

	<xsl:template match="as:Rules">
		<as:Rules>
			<as:default>
				<xsl:apply-templates select="as:default"/>
			</as:default>
			<xsl:if test="as:context">
			<as:context>
				<xsl:attribute name="condition"><xsl:value-of select="as:context/@condition"/></xsl:attribute>
				<xsl:apply-templates select="as:context"/>
			</as:context>
			</xsl:if>
		</as:Rules>
	</xsl:template>
	
	<xsl:template match="as:default">
		<as:context>
			<xsl:apply-templates select="as:context"/>
			
			<!-- Now add exclude rules from structure pass -->
			<xsl:variable name="excludeRules" select="$refactoredStructure//as:constraint"/>
			<xsl:for-each select="distinct-values($excludeRules//./@action)">
				<xsl:variable name="thisAction" select="."/>
				<xsl:variable name="actionRules" select="$excludeRules//.[@action = $thisAction]"/>
				<xsl:apply-templates select="$actionRules[1]"/>
			</xsl:for-each>
			
			<!-- xsl:message> total rules added: <xsl:value-of select="count($uniqueRules)"/></xsl:message -->
		</as:context>
	</xsl:template>
	
	<xsl:template match="as:context">
		<xsl:apply-templates mode="checkRules"/>
	</xsl:template>
	
	<xsl:template match="as:constraint">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="as:constraint" mode="outputStructure"/>
	
	<xsl:template match="as:constraint" mode="checkRules">
		<xsl:variable name="thisAction" select="@action"/><!-- store action so can adjust the XPath for reorder -->
		
		<!-- check to see if parent node renamed -->
		<xsl:choose>
			<xsl:when test="contains(./@action,'_CODE/') or(contains(./@action,'_COUNT/'))">
				<xsl:variable name="parentPath" select="concat(substring-before(substring-after(./@action,'//'),'/'),'_DETAIL')"/>
				<xsl:variable name="matchNode" select="count($refactoredStructure/as:Structure/AllRootLevelTypeDefinitions/node()[name()=$parentPath])"/>
				<xsl:choose>
					<xsl:when test="$matchNode &gt; 0">
						<xsl:element name="as:constraint">
							<xsl:if test="@condition"><xsl:attribute name="condition" select="@condition"/></xsl:if>
							<xsl:attribute name="action" select="replace(replace(@action,concat(replace($parentPath,'_DETAIL',''),'/'),concat($parentPath,'/')),'makeNillable','makeOptional')"/>
						</xsl:element>
					</xsl:when>
					<!-- Refactor makeNillable to makeOptional -->
					<xsl:when test="starts-with(./@action,'makeNillable(')">
						<xsl:element name="as:constraint">
							<xsl:attribute name="action" select="replace(@action,'makeNillable','makeOptional')"/>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="starts-with(./@action,'makeNillable(')"><!-- Refactor makeNillable to makeOptional -->
				<xsl:element name="as:constraint">
					<xsl:attribute name="action" select="replace(@action,'makeNillable','makeOptional')"/>
				</xsl:element>
			</xsl:when>
			<!-- reordering check - if parent node name not in XPath then insert name into XPath !!! -->
			<xsl:when test="contains(substring-before(replace(@action,'//','?'),','),'/')"><!-- See if the action path is still valid -->   
				<xsl:variable name="xpath" select="substring-after(substring-before(@action,','),'(')"/>
				<xsl:variable name="parentPath" select="substring-before(substring-after($xpath,'//'),'/')"/>
				<xsl:variable name="nodeName" select="substring-after($xpath,concat($parentPath,'/'))"/>
				<xsl:variable name="rootNode" select="count($parsedStructure/as:AssemblyStructure/as:Structure/node()[name()=$parentPath])"/>
				<xsl:variable name="parentNode" select="count($parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath])"/>
				<xsl:variable name="childNodes" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/node()/name()"/>
				<xsl:variable name="childNode">
					<xsl:for-each select="$childNodes">
						<xsl:if test=".=$nodeName"><xsl:value-of select="$nodeName"/></xsl:if>
					</xsl:for-each>
				</xsl:variable>
				<!-- xsl:variable name="childNode" select="count($refactoredStructure/as:Structure[*/*/node()/name()=$nodeName][*/node()/name()=$parentPath])"/ -->
				<xsl:choose>
					<xsl:when test="$parentPath = $nodeName"/><!-- ignore child and parent same path -->
					<xsl:when test="$parentNode &gt; 0 and($childNode='')">
						<xsl:variable name="findNodes" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/*/node()/name()"/>
						<xsl:variable name="findNodesContent" select="$parsedStructure/as:AssemblyStructure/as:Structure/*/node()[name()=$parentPath]/*/node()"/>
						<xsl:variable name="findNode">
							<xsl:for-each select="$findNodes">
								<xsl:variable name="this" select="position()"/>
								<xsl:if test=".=$nodeName"><xsl:value-of select="$nodeName"/> - <xsl:value-of select="$findNodesContent[position()=$this]/text()"/> - <xsl:value-of select="$this"/> + <xsl:value-of select="$findNodesContent[position()=$this]/parent::node()/name()"/></xsl:if>
							</xsl:for-each>
						</xsl:variable>
						<xsl:choose>
							<xsl:when test="not($findNode='') and(contains($findNode,'%'))">
								<xsl:message> new parent node of rule <xsl:value-of select="$xpath"/> is <xsl:value-of select="substring-after($findNode,'+ ')"/></xsl:message>
								<xsl:element name="as:constraint">
									<xsl:if test="@condition"><xsl:attribute name="condition" select="@condition"/></xsl:if>
									<xsl:attribute name="action" select="replace(@action,$xpath,concat('//',$parentPath,'/',substring-after($findNode,'+ '),'/',$nodeName))"></xsl:attribute>
								</xsl:element>
							</xsl:when>
							<xsl:otherwise>
								<xsl:apply-templates select="."/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="."/>
					</xsl:otherwise>
				</xsl:choose>  
			</xsl:when>
			<xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy>
			<xsl:if test="string-length($include) &gt; 0">
				<xsl:attribute name="as:include" select="$include"/>
			</xsl:if>
			<xsl:apply-templates mode="start"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="@*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:include" mode="start">
		<!-- If there are any includes expand them to inline structure components (TBD the CAMeditor may already do this in Java -->
		<!-- TBD This part may need attention -->
		<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/>/<xsl:value-of select="."/></xsl:variable>
		<xsl:variable name="include" select="document($filePath)"/>
		<xsl:apply-templates select="$include/*" mode="start">
			<xsl:with-param name="include" select="."/>
		</xsl:apply-templates>		
		
	</xsl:template>
	
	<xsl:template match="as:include">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="*" mode="structure">
		<xsl:param name="parent"/>
		<xsl:param name="only_one"/>
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="parentItem" select="if(../name()='as:Structure') then '' else ../name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parentPath" select="replace(concat('/',$parentItem,$contextMatch,'/'),'//','/')"/>
		<!-- find virtual parent path, this is the parent in the dictionary that precedes this item to be expanded -->
		<xsl:if test="$debug &gt; 0"><xsl:message>- <xsl:value-of select="$thisItem"/>[<xsl:value-of select="$parentItem"/>][<xsl:value-of select="$rootElement"/>]</xsl:message></xsl:if>
		
		<xsl:variable name="noNSname" select="if(contains($thisItem,':')) then substring-after($thisItem,':') else $thisItem"/>
		<xsl:variable name="noNSmatch" select="concat('/',$noNSname)"/>
		
		<xsl:variable name="rootNode" select="if(parent::*/name()='AllRootLevelTypeDefinitions') then true() else false()"/>
		<xsl:if test="$rootNode = true()"><xsl:message> - parent <xsl:value-of select="name()"/></xsl:message></xsl:if>
		
		<xsl:variable name="nodeItemNames" select="child::*/following-sibling::*/name()"/>
		<xsl:variable name="duplicateItemNames">
			<xsl:for-each select="distinct-values($nodeItemNames)">
				<xsl:variable name="thisName" select="."/>
				<xsl:if test="count($nodeItemNames[.=$thisName]) &gt; 1">
					<xsl:variable name="positionCount"><xsl:for-each select="$nodeItemNames"><xsl:if test=".=$thisName"><xsl:value-of select="concat(position() + 2,'|')"/></xsl:if></xsl:for-each></xsl:variable>
					<xsl:value-of select="concat(.,'|',substring-before($positionCount,'|'),'|')"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:variable name="removeDuplicates">
			<xsl:if test="not($duplicateItemNames='') and(not(concat('|',$duplicateItemNames) = $only_one))">
				<xsl:value-of select="concat('|',$duplicateItemNames)"/>
				<xsl:if test="$debug &gt; 0"><xsl:message> - only one: <xsl:value-of select="parent::*/name()"/> + <xsl:value-of select="$duplicateItemNames"/></xsl:message></xsl:if>
			</xsl:if>
		</xsl:variable>
		<xsl:if test="not($only_one='') and($debug &gt; 1)"><xsl:message> - removing <xsl:value-of select="$only_one"/></xsl:message></xsl:if>
		<!-- Create structure header, or process child nodes  -->
		<xsl:choose>
			<xsl:when test="name()='as:Structure'">
				<xsl:variable name="actualRootNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>as:Structure root: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				<xsl:element name="{name()}" namespace="{$actualRootNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:message>- structure of: <xsl:value-of select="./*/name()"/></xsl:message>
					<xsl:variable name="newStructure" select="./*"/>
					<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="./*/name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
					<xsl:element name="{./*/name()}" namespace="{$actualNS}"><!-- Insert root element, then process children -->
						<xsl:apply-templates select="$newStructure/attribute::*" mode="insert"/>
						<xsl:apply-templates mode="structure" select="$newStructure/./*"/>
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:when test="contains($only_one,concat('|',name(),'|')) and(position() &lt; xsd:integer(substring-before(substring-after($only_one, concat('|',name(),'|')),'|')))">
				<!-- Need only one of these -->
				<xsl:message> - REMOVED - <xsl:value-of select="concat($parent,' / ',name())"/> - <xsl:value-of select="position()"/></xsl:message>
				<!-- xsl:apply-templates mode="structure">
					<xsl:with-param name="parent" select="$parent"/>
					<xsl:with-param name="only_one" select="replace($only_one,concat('\|',name(),'\|[0-9]{1,4}\|'),'')"/>
				</xsl:apply-templates -->
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>structure here: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				
				<!-- If element contains child of same name code, then rename parent with _DETAIL -->
				
				<xsl:variable name="nodeNameChk" select="name()"/>
				
				<xsl:variable name="checkChildNames">
				<xsl:if test="count(child::*) &gt; 0">
					<xsl:for-each select="child::*">
						<xsl:if test="name()=$nodeNameChk and(contains($nodeNameChk,'_CODE') or(contains($nodeNameChk,'_COUNT')))">true</xsl:if>
					</xsl:for-each>
				</xsl:if>
				</xsl:variable>
				
				<xsl:if test="contains($checkChildNames,'true') and($debug &gt; 0)"><xsl:message> - child match! </xsl:message></xsl:if>
				
				<xsl:variable name="nodeName" select="if(contains($checkChildNames,'true')) then concat(name(),'_DETAIL') else name()"/>
				
				<xsl:variable name="annoMatches" select="$itemAnnotations//as:Extension/camed:annotation[@item=concat('//',$parentItem,$contextMatch)]"/>
				
				<!-- If not marked as duplicate item write this structure item out -->
				
				<xsl:element name="{$nodeName}" namespace="{$actualNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:if test="not(exists(*)) and string-length(text()) &gt; 0"><xsl:value-of select="text()"/></xsl:if><!-- insert default value text -->
					<xsl:if test="as:include">
						<xsl:apply-templates mode="include"/>
					</xsl:if>
					<xsl:if test="$rootNode = false() and($debug &gt; 0)"><xsl:message> - item <xsl:if test="count(child::*) &gt; 0"> - <xsl:value-of select="count(child::*)"/></xsl:if> : <xsl:value-of select="name()"/> - <xsl:value-of select="position()"/></xsl:message></xsl:if>

					<xsl:choose>
						<xsl:when test="count(child::*) = 0 and(string-length(text()) = 0)">
							<xsl:if test="$debug &gt; 0"><xsl:message> EMPTY! </xsl:message></xsl:if>
							<xsl:element name="as:constraint">
								<xsl:attribute name="action" select="concat('excludeTree(//',if($parent='') then parent::*/name() else $parent,'/',name(),')')"/>
							</xsl:element>
						</xsl:when>
						
						<xsl:when test="count(child::*) &gt; 0 and(ends-with($parent,'_CODE') or(ends-with($parent,'_CODE_DETAIL')))">
							<xsl:if test="$debug &gt; 0"><xsl:message> Exclude MODEL! </xsl:message></xsl:if>
							<xsl:element name="as:constraint">
								<xsl:attribute name="action" select="concat('excludeTree(//',if($parent='') then parent::*/name() else $parent,'/',name(),')')"/>
							</xsl:element>
						</xsl:when>
						<xsl:when test="$annoMatches//camed:documentation[@type='Ref']">
							<xsl:element name="as:constraint">
								<xsl:attribute name="action" select="concat('excludeTree(//',if($parent='') then parent::*/name() else $parent,'/',name(),')')"/>
							</xsl:element>
						</xsl:when>
						<xsl:when test="$annoMatches//camed:documentation[@type='BaseRef']">
							<xsl:element name="as:constraint">
								<xsl:attribute name="action" select="concat('excludeTree(//',if($parent='') then parent::*/name() else $parent,'/',name(),')')"/>
							</xsl:element>
						</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
					
					<xsl:apply-templates mode="structure">
						<xsl:with-param name="parent" select="$nodeName"/>
						<xsl:with-param name="only_one" select="if($removeDuplicates = '') then $only_one else $removeDuplicates"/>
					</xsl:apply-templates>
				</xsl:element>
				
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="text()" mode="structure"/>
	
	<xsl:template match="text()" mode="structureRules"/>
	
	<xsl:template match="@*" mode="insert">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:attribute name="{name()}" namespace="{$namespace-uri}"><xsl:value-of select="."/></xsl:attribute>
	</xsl:template>

	<xsl:template match="*|@*|text()">
	</xsl:template>

	<xsl:template match="*" mode="rule">
		<xsl:param name="action"/>

		<xsl:copy-of select="."/>
		
	</xsl:template>
	
	<!-- These templates are from original code -->
	
	<xsl:template match="as:constraint|as:context|@*" mode="startRules">
		<xsl:variable name="action" select="@action"/>
		<xsl:choose>
			<xsl:when test="contains($action,'applyTemplate')">
				<xsl:variable name="templateName" select="if (contains(substring-after($action,','),',')) then substring-before(substring-after($action,','),',') else substring-before(substring-after($action,','),')')"/>
				<xsl:variable name="templateFileName" select="concat('file://',$inputFilePath,'/',$templateName)"/>
				<!-- xsl:message>templateFileName=<xsl:value-of select="$templateFileName"/></xsl:message -->
				<xsl:variable name="template" select="document($templateFileName)"/>
				<xsl:variable name="applyTemplateRules" select="$template//as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint"/>
				<xsl:apply-templates select="$applyTemplateRules" mode="start"/>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*" mode="structureRules">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		
		<!-- Find the names of the child items of this node -->
		<xsl:variable name="theseChildrenMembers" select="child::node()"/>
		<xsl:variable name="ChildElementNames">
			<xsl:for-each select="$theseChildrenMembers/.">|<xsl:value-of select="name()"/>|</xsl:for-each>
		</xsl:variable>
		
		<xsl:variable name="childNodesCount" select="count(tokenize($ChildElementNames,'\|\|'))"/>
		
		<!-- Choose to lookup item structure in dictionary, or simply skip  -->
		
		<xsl:apply-templates mode="structureRules"/>
		
	</xsl:template>
	
	<xsl:template match="@*" mode="structureRules">
		<xsl:param name="context"/>
		<xsl:variable name="findAttrPath" select="concat($context,name())"/>
		<xsl:for-each select="//Property[ends-with(@context,$findAttrPath)][1]">
			<xsl:if test="contains(./@cardinality,'optional')">
				<as:constraint action="{concat('makeOptional(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:if test="contains(./@cardinality,'repeatable')">
				<as:constraint action="{concat('makeRepeatable(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:call-template name="setFacetRules">
				<xsl:with-param name="type" select="@type"/>
				<xsl:with-param name="mask" select="@mask"/>
				<xsl:with-param name="path" select="concat('/',@context)"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="setFacetRules">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:param name="path"/>
		<xsl:variable name="thisPath" select="if(contains($path,'/@')) then $path else replace($path,'@','/@')"/>
		<xsl:choose>
			<xsl:when test="$type='ID'"><as:constraint action="{concat('datatype(',$thisPath,',ID)')}"/></xsl:when>
			<xsl:when test="$type='IDREF'"><as:constraint action="{concat('datatype(',$thisPath,',IDREF)')}"/></xsl:when>
			<xsl:when test="$type='IDREFS'"><as:constraint action="{concat('datatype(',$thisPath,',IDREFS)')}"/></xsl:when>
			<xsl:when test="$type='any'"><as:constraint action="{concat('datatype(',$thisPath,',any)')}"/></xsl:when>
			<xsl:when test="$type='anyURI'"><as:constraint action="{concat('datatype(',$thisPath,',anyURI)')}"/></xsl:when>
			<xsl:when test="$type='base64Binary'"><as:constraint action="{concat('datatype(',$thisPath,',base64Binary)')}"/></xsl:when>
			<xsl:when test="$type='anyAttribute'"><as:constraint action="{concat('datatype(',$thisPath,',anyAttribute)')}"/></xsl:when>
			<xsl:when test="$type='boolean'"><as:constraint action="{concat('datatype(',$thisPath,',boolean)')}"/></xsl:when>
			<xsl:when test="$type='integer'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='decimal'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='float'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='unsignedInt'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask='') then '########' else $mask,')')}"/></xsl:when>
			<xsl:when test="$type='unsignedLong'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask='') then '############' else $mask,')')}"/></xsl:when>
			<xsl:when test="$type='date'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='time'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='list'"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask,',','''|'''),''')')}"/></xsl:when>
			<xsl:when test="contains($mask,'|')"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask,'\|','''|'''),''')')}"/></xsl:when>
			<xsl:when test="$type='datetime'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="setTypeSample">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:choose>
			<xsl:when test="$type=''"/>
			<xsl:when test="$type='date'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='datetime'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='time'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='integer'">%1234%</xsl:when>
			<xsl:when test="$type='decimal'">%654.10%</xsl:when>
			<xsl:when test="$type='float'">%7654.0001%</xsl:when>
			<xsl:when test="$type='anyURI'">%http://www.camtoolkit.com%</xsl:when>
			<xsl:otherwise>%<xsl:value-of select="$type"/>%</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="nsResolver">
		<xsl:param name="itemName"/>
		<xsl:param name="ns"/>
		<xsl:choose>
			<xsl:when test="count(tokenize($ns,'http:')) &gt; 1">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:value-of select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$ns=''">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:variable name="matchNS" select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
						<xsl:choose>
							<xsl:when test="string-length($matchNS) &lt; 1"><xsl:value-of select="concat('http://jcam.org.uk/ns/',$thisNS)"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$matchNS"/></xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="$ns"/></xsl:otherwise>
		</xsl:choose>	
	</xsl:template>
	
	<xsl:template match="*" mode="structureAnno">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		<!-- Generate annotation for code detail items  -->	
		<xsl:if test="ends-with($thisItem,'_CODE_DETAIL')">
			<camed:annotation item="{concat('//AllRootLevelTypeDefinitions/',$thisItem)}">
				<camed:documentation type="Definition">Details for <xsl:value-of select="$thisItem"/>.</camed:documentation>
			</camed:annotation>
		</xsl:if>
									
		<xsl:apply-templates mode="structureAnno"/>
		
	</xsl:template>
	
	<xsl:template match="*" mode="importAnno">
		<xsl:param name="this"/>
		<xsl:param name="parent"/>
		<xsl:param name="rest"/>
		<xsl:variable name="thisItem" select="$this/./@item"/>
		<xsl:variable name="thisContext" select="$this/./@context"/>		
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="more" select="count($rest/Property)"/>
		<!-- xsl:message>anno: [<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$more"></xsl:value-of>]</xsl:message -->
		<xsl:variable name="parentContext" select="concat($this/./@context,'/')"/>
		<xsl:variable name="shortContext">
			<xsl:call-template name="shortContext">
				<xsl:with-param name="thisContext" select="$thisContext"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:choose>
			<!-- xsl:when test="$this/./@CCTS ='ASBIE'"/ --><!-- ignore attributes -->
			<xsl:when test="$more &gt; 0">
				<!-- Now insert the annotation from the definition details -->
				
				<xsl:for-each select="$rest/Property">
					<!-- xsl:message>[<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$parentContext"/>][<xsl:value-of select="@context"/>]</xsl:message -->
					<!-- Now see if there are children of this element -->
					<xsl:variable name="chkContext" select="concat(@context,'/')"/>
					<xsl:variable name="itemContext" select="substring-after(@context,$parentContext)"/>
					<xsl:variable name="next-namespace-uri" select="namespace-uri()"/>
					<xsl:variable name="children" select="count($rest/Property[starts-with(@context,$chkContext)])"/>
					<xsl:choose>
						<xsl:when test="$children &gt; 0">
							<!-- Process parent and children -->
							<xsl:variable name="importRest">
								<xsl:for-each select="$rest/Property[contains(@context,$chkContext)]">
									<xsl:copy-of select="."/>
								</xsl:for-each>
							</xsl:variable>
							<xsl:variable name="uniqueAnnoList" select="distinct-values($importRest/Property/@context)"/>
							<xsl:variable name="importRestUnique">
								<xsl:for-each select="$uniqueAnnoList">
									<xsl:variable name="thisContextItem" select="."/>
									<xsl:copy-of select="$importRest/Property[@context=$thisContextItem][1]"/>
								</xsl:for-each>
							</xsl:variable>
							<xsl:apply-templates select="." mode="importAnno">
								<xsl:with-param name="this" select="."/>
								<xsl:with-param name="parent" select="$thisItem"/>
								<xsl:with-param name="rest" select="$importRestUnique"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="contains($itemContext,'/')"><!-- Ignore child of parent item --></xsl:when>
						<xsl:otherwise><!-- Process this singleton element -->
							<xsl:if test="string-length(./@definition) &gt; 0">
								<xsl:variable name="shortContext">
									<xsl:call-template name="shortContext">
										<xsl:with-param name="thisContext" select="concat('/',./@context)"/>
										<xsl:with-param name="parent" select="$parent"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
									<xsl:attribute name="item" select="$shortContext"/>
									<camed:documentation type="Definition"><xsl:value-of select="./@definition"/></camed:documentation>
									<camed:documentation type="Expanded">From <xsl:value-of select="./@item"/> dictionary definition.</camed:documentation>
								</xsl:element>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				
			</xsl:when>
			<xsl:otherwise>
				<!-- Process this singleton element -->
					<xsl:variable name="shortContext">
						<xsl:call-template name="shortContext">
							<xsl:with-param name="thisContext" select="concat('/',$this/./@context)"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
						<xsl:attribute name="item" select="$shortContext"/>
						<xsl:if test="string-length($this/./@definition) &gt; 0"><camed:documentation type="Definition"><xsl:value-of select="$this/./@definition"/></camed:documentation></xsl:if>
						<camed:documentation type="Expanded">From <xsl:value-of select="$this/./@item"/> dictionary definition.</camed:documentation>
					</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="shortContext">
		<xsl:param name="thisContext"/>
		<xsl:param name="parent"/>
		<xsl:variable name="contextPathNodes" select="tokenize($thisContext,'/')"/>
		<xsl:variable name="contextPathCount" select="count($contextPathNodes)"/>
		<xsl:variable name="shortContext">
			<xsl:choose>
				<xsl:when test="$contextPathCount &gt; 3"><xsl:value-of select="concat('//',$contextPathNodes[$contextPathCount - 1],'/',$contextPathNodes[$contextPathCount])"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('//',replace(concat($parent,$thisContext),'//','/'))"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="if(contains($shortContext,'@') and not(contains($shortContext,'/@'))) then replace($shortContext,'@','/@') else $shortContext"/>
	</xsl:template>
	
	<xsl:template match="*" mode="outputStructure">
		<xsl:element name="{name()}">
			<xsl:apply-templates mode="outputStructure" select="@*"/>
			<xsl:apply-templates mode="outputStructure"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="@*" mode="outputStructure">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="*" mode="reorder">
		<xsl:message>Start item re-ordering</xsl:message>
		<xsl:for-each select="$itemReorder/as:Reordering//as:Group/*[1]/name()"><xsl:message> - group <xsl:value-of select="."/></xsl:message></xsl:for-each>
		<xsl:copy>
			<xsl:copy-of select="@*"/>
			<xsl:element name="{name(child::*[1])}">
				<xsl:for-each select="child::*[1]/child::*">
					<xsl:variable name="thisItem" select="name(.)"/>
					<xsl:choose>
						<xsl:when test="$itemReorder//as:Reordering/as:Group/child::*[1]/name()=$thisItem"><!-- Reordering at parent node level -->
							<xsl:message> - reorder <xsl:value-of select="$thisItem"/></xsl:message>
							<xsl:variable name="reorderDetail" select="$itemReorder//as:Reordering/as:Group[child::*[1]/name()=$thisItem]/*"/>
							<xsl:element name="{$reorderDetail/./name()}">
								<xsl:copy-of select="$reorderDetail/./child::*"/>
								<xsl:apply-templates select="./child::*" mode="reordering">
									<xsl:with-param name="parent" select="$thisItem"/>
									<xsl:with-param name="detailItems" select="$reorderDetail"/>
								</xsl:apply-templates>
							</xsl:element>
						</xsl:when>
						<xsl:when test="not($thisItem='')"><!-- Check for reordering of every parent node level children -->
							<xsl:message> - item <xsl:value-of select="$thisItem"/></xsl:message>
							<xsl:copy>
								<xsl:copy-of select="@*"/>
								<xsl:copy-of select="text()"/>
								<xsl:apply-templates select="*" mode="reordering">
									<xsl:with-param name="parent" select="''"/>
									<xsl:with-param name="detailItems" select="/"/>
								</xsl:apply-templates>
							</xsl:copy>
						</xsl:when>
						<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise><!-- Default action -->
					</xsl:choose>
				</xsl:for-each>
			</xsl:element>
		</xsl:copy>
		<xsl:message>Done item re-ordering</xsl:message>
	</xsl:template>
	
	<xsl:template match="*" mode="reordering">
		<xsl:param name="parent"/>
		<xsl:param name="detailItems"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="thisParent" select="../name()"/>
		<xsl:choose>
			<xsl:when test="$parent=$thisParent and($detailItems//.[name(.)=$thisItem])"><xsl:if test="$debug &gt; 0"><xsl:element name="skipped"><xsl:value-of select="$thisItem"/></xsl:element></xsl:if></xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:copy-of select="@*"/>
					<xsl:copy-of select="text()"/>
					<xsl:choose>
						<xsl:when test="./as:constraint"><xsl:copy-of select="./child::*"/></xsl:when><!-- Skip items marked for excluding and just insert them here -->
						<xsl:when test="$itemReorder//as:Reordering/as:Group/child::*[1]/name()=$thisItem"><!-- Embedded reordering of child components -->
							<xsl:message> -- reorder <xsl:value-of select="$thisItem"/></xsl:message>
							<xsl:variable name="reorderDetail" select="$itemReorder//as:Reordering/as:Group[child::*[1]/name()=$thisItem]/*"/>
							<xsl:copy-of select="$reorderDetail/./child::*"/>
							<xsl:apply-templates select="./child::*" mode="reordering">
								<xsl:with-param name="parent" select="$thisItem"/>
								<xsl:with-param name="detailItems" select="$reorderDetail"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="child::*" mode="reordering">
								<xsl:with-param name="parent" select="$parent"/>
								<xsl:with-param name="detailItems" select="$detailItems"/>
							</xsl:apply-templates>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
</xsl:stylesheet>